/*
 * testapp.cpp
 *
 * Demonstrates use of CTaskBarIcon class
 *
 */

#include <windows.h>
#include <stdio.h>

#include <taskbar.h>

#include "testapp.h"
#include "resource.h"

#ifdef WIN32
#define _EXPORT 
#else
#define _EXPORT __export
#endif

HANDLE		MyInstance;
HWND		g_hWnd;
static char	GenericClass[32]="GenericClass";

static BOOL FirstInstance( HANDLE );
static BOOL AnyInstance( HANDLE, int, LPSTR );

long _EXPORT FAR PASCAL WindowProc( HWND, unsigned, UINT, LONG );

MyTaskBarIcon* g_taskBarIcon = NULL;

/*
 * WinMain - initialization, message loop
 */
int PASCAL WinMain( HANDLE this_inst, HANDLE prev_inst, LPSTR cmdline,
		    int cmdshow )
{
    MSG		msg;

    SetLastError(0);

    MyInstance = this_inst;
    if( !prev_inst ) {
	if( !FirstInstance( this_inst ) ) return( FALSE );
    }
    if( !AnyInstance( this_inst, cmdshow, cmdline ) ) return( FALSE );
    
    while( GetMessage( &msg, NULL, NULL, NULL ) ) {

	TranslateMessage( &msg );
	DispatchMessage( &msg );

    }

	delete g_taskBarIcon;

    return( msg.wParam );

} /* WinMain */

/*
 * FirstInstance - register window class for the application,
 *		   and do any other application initialization
 */
static BOOL FirstInstance( HANDLE this_inst )
{
    WNDCLASS	wc;
    BOOL	rc;

    /*
     * set up and register window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = (WNDPROC) WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof( DWORD );
    wc.hInstance = this_inst;
    wc.hIcon = LoadIcon( this_inst, "TestAppIcon" );
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = GetStockObject( WHITE_BRUSH );
    wc.lpszMenuName = "GenericMenu";
    wc.lpszClassName = GenericClass;
    rc = RegisterClass( &wc );
    return( rc );

} /* FirstInstance */

/*
 * AnyInstance - do work required for every instance of the application:
 *		  create the window, initialize data
 */
static BOOL AnyInstance( HANDLE this_inst, int cmdshow, LPSTR cmdline )
{
	HICON hIcon = ::LoadIcon(this_inst, "TESTAPPICON");
	g_taskBarIcon = new MyTaskBarIcon;
	if (!g_taskBarIcon->IsOK())
	{
		MessageBox(NULL, "Could not create CTaskBarIcon.", "CTaskBarIcon Sample", MB_OK|MB_ICONEXCLAMATION);
	}
	else
	{
		if (!g_taskBarIcon->SetIcon(hIcon))
		{
			MessageBox(NULL, "Could not set icon in taskbar.", "CTaskBarIcon Sample", MB_OK|MB_ICONEXCLAMATION);
		}
	}
	if (hIcon)
		::DeleteObject(hIcon);

    /*
     * create main window
     */
    g_hWnd = CreateWindowEx(0,
	GenericClass,		/* class */
	"CTaskBarIcon Sample",	/* caption */
	WS_OVERLAPPEDWINDOW,	/* style */
	CW_USEDEFAULT,		/* init. x pos */
	CW_USEDEFAULT,		/* init. y pos */
	CW_USEDEFAULT,		/* init. x size */
	CW_USEDEFAULT,		/* init. y size */
	NULL,			/* parent window */
	NULL,			/* menu handle */
	this_inst,		/* program handle */
	NULL			/* create parms */
	);
		    
    if( !g_hWnd ) return( FALSE );

    /*
     * set up data associated with this window
     */

    /*
     * display window
     */
/* Don't display window until user has double-clicked on
 * task bar icon
    ShowWindow( hwnd, cmdshow );
    UpdateWindow( hwnd );
*/

    return( TRUE );
                        
} /* AnyInstance */

/*
 * AboutDlgProc - processes messages for the about dialog.
 */
BOOL _EXPORT FAR PASCAL AboutDlgProc( HWND hwnd, unsigned msg,
				UINT wparam, LONG lparam )
{
    lparam = lparam;			/* turn off warning */

    switch( msg ) {
    case WM_INITDIALOG:
	return( TRUE );

    case WM_COMMAND:
    	if( LOWORD( wparam ) == IDOK ) {
	    EndDialog( hwnd, TRUE );
	    return( TRUE );
	}
	break;
    }
    return( FALSE );

} /* AboutDlgProc */

/*
 * WindowProc - handle messages for the main application window
 */
LONG _EXPORT FAR PASCAL WindowProc( HWND hwnd, unsigned msg,
				     UINT wparam, LONG lparam )
{
    DLGPROC 	proc;

    switch( msg )
    {
    case WM_COMMAND:
		{
			switch( LOWORD( wparam ) )
			{
			case MENU_ABOUT:
				{
					proc = (DLGPROC)MakeProcInstance( (DLGPROC)AboutDlgProc, MyInstance );
					DialogBox( MyInstance,"AboutBox", hwnd, proc );
					FreeProcInstance( proc );
					break;
				}
				
			case MENU_EXIT:
				{
					PostQuitMessage(0);
					break;
				}

			case ID_TIP1:
				{
                    // Set the tooltip, only.
                    g_taskBarIcon->SetIcon((HICON) 0, "Tooltip Number One");
					break;
				}

			case ID_TIP2:
				{
                    // Set the tooltip, only.
                    g_taskBarIcon->SetIcon((HICON) 0, "Tooltip Number Two");
					break;
				}
			}
        }
		break;

    case WM_DESTROY:
		PostQuitMessage( 0 );
		break;
		
	case WM_SYSCOMMAND:
		{
			if (wparam == SC_MINIMIZE)
			{
				::ShowWindow(g_hWnd, SW_HIDE);
				return 0;
			}
			else
			{
				return DefWindowProc(hwnd, msg, wparam, lparam);
			}
			break;
		}
    default:
		return( DefWindowProc( hwnd, msg, wparam, lparam ) );
    }
    return( 0L );

} /* WindowProc */

// Overridables
void MyTaskBarIcon::OnMouseMove(void)
{
}

void MyTaskBarIcon::OnLButtonDown(void)
{
}

void MyTaskBarIcon::OnLButtonUp(void)
{
}

void MyTaskBarIcon::OnRButtonDown(void)
{
}

void MyTaskBarIcon::OnRButtonUp(void)
{
}

void MyTaskBarIcon::OnLButtonDClick(void)
{
	::ShowWindow(g_hWnd, SW_SHOW);
}

void MyTaskBarIcon::OnRButtonDClick(void)
{
}

